###############################################################################
#
# itunes_remove_by_location.pl
#
# This script will find all tracks in the user-selected playlist whose file
# path matches a user-supplied pattern (regex).  Matching tracks are removed
# from the playlist.
#
# Copyright (C) 2007  Robert Jacobson
# written by: Robert Jacobson (http://mysite.verizon.net/teridon/itunesscripts)
# Last Updated: 30 July 2007
# Version 1.1
#
# This script is GPL v2.  see http://www.gnu.org/copyleft/gpl.html
#
#	1.1 Update loop logic to start at end of list and step backwards through items.
#		Hopefully this will avoid a possible race condition
#
###############################################################################

sub itunes_remove_by_location {
	
	use strict;
	use Win32::OLE;
	
	## Create the OLE Object
	my $iTunes = Win32::OLE->new('iTunes.Application') or die Win32::OLE->LastError();
	
	print "Enter pattern/regex to search for: ";
	chomp (my $pattern = <STDIN>);
	
	# Get the possible sources
	my $sources = $iTunes->Sources();
	my $sourcesCount = $sources->Count();
	my $source = '';
	my $sourceKind = '';
	
	my $n = 1;
	my $deleted_tracks = 0;
	
	# For each source, figure out kind	
	for ($n = 1; $n <= $sourcesCount; $n++) {
		$source = $sources->Item($n);
		$sourceKind = $source->Kind();
		
		if ($sourceKind == 1) {
			# Get the playlists in the Library
			my $playlists = $source->Playlists();
			my $num_playlists = $playlists->Count();
			print "There are $num_playlists playlists\n";
			
			# For each playlist, show the name and number of tracks
			for (my $j = 1 ; $j <= $num_playlists; $j++) {
				my $playlist = $playlists->Item($j);
				my $playlist_name = $playlist->Name();
				print "\t$j : $playlist_name\n";
			}
			
			print "Enter comma-separated playlist numbers: ";
			chomp (my $nums = <STDIN>);
			my @nums = split(/,/ , $nums);
			for my $i (@nums) {
				my $playlist = $playlists->Item($i);
				my $playlist_name = $playlist->Name();
				
				my $tracks = $playlist->Tracks;
				my $num_tracks = $tracks->Count();
				print "You selected $playlist_name, which has $num_tracks tracks\n";
				
				# Get all the tracks in the playlist
				for (my $k = $tracks->Count ; $k > 0 ; $k-- ) {
					my $track = $tracks->Item($k);
					my $track_kind = $track->Kind();
	
					if ($track_kind == 1) {
						# File
						my $path = $track->Location();
						if ($path =~ /$pattern/) {
							my $songname = $track->Name();
							print "Match: song $songname , path is $path\n";
							$track->Delete();
							$deleted_tracks++;
						}
					}
				}
			}
		}
	}

	print "Done.  Removed $deleted_tracks tracks\n";

}


1;
