###############################################################################
#
# itunes_remove_by_playlist.pl
#
# This script will remove all songs from your iTunes Library based on the
# playlist you selected.  It will also optionally remove those files from
# your hard drive.
#
# Copyright (C) 2007  Robert Jacobson
# written by: Robert Jacobson (http://mysite.verizon.net/teridon/itunesscripts)
# Last Updated: 30 July 2007
# Version 1.1
#
# This script is GPL v2.  see http://www.gnu.org/copyleft/gpl.html
#
#	1.1 Update loop logic to start at end of list and step backwards through items.
#		Hopefully this will avoid a possible race condition
#
###############################################################################

sub itunes_remove_by_playlist {

	use strict;
	use Win32::OLE;
	#use Data::Dumper;	# For fancy printing of structures
	
	$Data::Dumper::Sortkeys = 1;
	
	# Script Info
	#print <<ENDDATA;
	#
	#**************************************************************
	#This script will delete all tracks from the iTunes Library
	#for any playlist(s) you select.
	#It will also OPTIONALLY delete the files from your hard drive
	#
	#This script will run MUCH faster if you select a playlist to
	#view in iTunes OTHER THAN the Library or the playlist(s) you
	#are deleting (e.g. if you are deleting playlist "foo", select
	#playlist "bar" in iTunes)
	#
	#Cancel the script at any time using Ctrl-C!!
	#**************************************************************
	#
	#ENDDATA
	
	#sleep 2;
	
	## Create the OLE Object
	my $iTunes = Win32::OLE->new('iTunes.Application') or die Win32::OLE->LastError();
	
	# Get the possible sources
	my $sources = $iTunes->Sources();
	my $sourcesCount = $sources->Count();
	my $source = '';
	my $sourceKind = '';
	my $library = $iTunes->LibraryPlaylist();
	
	my %Library_IDs;
	
	# Retrieve Library information for all tracks
	$| = 1;
	print "Reading Library...";
	my $tracks = $library->Tracks();
	
	my $num_tracks = $tracks->Count();
	#print "\t$num_tracks tracks\n";
	
	
	# Get all the tracks in the playlist
	for (my $k = 1 ; $k <= $tracks->Count ; $k++ ) {
		my $track = $tracks->Item($k);
		my $track_kind = $track->Kind();
	
		if ($track_kind == 1) {
			# Get the TrackDatabase ID, and store it for later
			my $id = $track->TrackDatabaseID;
			$Library_IDs{$id} = $track;
		}
	
	}
	
	print "Done.\n";
	
	
	# Now allow user to choose a playlist 
	
	my $n = 1;
	print "There are " . $sourcesCount . " sources currently available\n";
	
	# For each source, figure out kind	
	for ($n = 1; $n <= $sourcesCount; $n++) {
		$source = $sources->Item($n);
		$sourceKind = $source->Kind();
		
		if ($sourceKind == 1) {
			# Get the playlists in the Library
			my $playlists = $source->Playlists();
			my $num_playlists = $playlists->Count();
			print "There are $num_playlists playlists\n";
			
			# For each playlist, show the name and number of tracks
			for (my $j = 1 ; $j <= $num_playlists; $j++) {
				my $playlist = $playlists->Item($j);
				my $playlist_name = $playlist->Name();
				print "\t$j : $playlist_name\n";
			}
			
			print "Enter comma-separated playlist numbers: ";
			chomp (my $nums = <STDIN>);
			my @nums = split(/,/ , $nums);
			# Create a temporary Playlist to select while tracks are being deleted
			# This speeds up the operation		
			my $old_playlist = $iTunes->BrowserWindow->SelectedPlaylist;
			my $tmp_playlist = $iTunes->CreatePlaylist("Temporary Playlist $$");
			
			# For some reason I have to use LetProperty() instead of
			# $iTunes->BrowserWindow->{SelectedPlaylist} = $tmp_playlist;
			$iTunes->BrowserWindow->LetProperty('SelectedPlaylist', $tmp_playlist);
	
			my $delete = "x";
			while ($delete !~ /^[yn]/i) {
				print "Do you want to delete files from the hard drive? [y/n] ";
				chomp ($delete = <STDIN>);
			}
	
			if ($delete eq "y") {
				print "*********** OK, will delete files!\n";
				print "*********** Please NOTE: this script will not delete the\n";
				print "*********** folders containing the files, even if they are\n";
				print "*********** empty\n";
			} elsif ($delete eq "n") {
				print "OK, will NOT delete files\n";
			}
			
			for my $i (@nums) {
				my $playlist = $playlists->Item($i);
				my $playlist_name = $playlist->Name();
				print "You selected $playlist_name\n";
				
				if ($playlist_name eq "Library") {
					print "You want to delete your entire Library?!  I don't think so...  Exiting!\n";
					return;
				}
				
				my $tracks = $playlist->Tracks;
				my $num_tracks = $tracks->Count();
				print "\t$num_tracks tracks\n";
				
				# Get all the tracks in the playlist
				for (my $k = $tracks->Count ; $k > 0 ; $k-- ) {
					my $track = $tracks->Item($k);
					#print Dumper $track;
					#my $foo = <STDIN>; #pause 
					my $track_kind = $track->Kind();
	
					if ($track_kind == 1) {
						my $id = $track->TrackDatabaseID;
						my $location = $track->Location();
						#print Dumper $track;
						# Get the track info from the Library
						my $library_track = $Library_IDs{$id};
						#print "Library Track:\n" . Dumper $library_track;
						# Now remove it from the Library
						$library_track->Delete();
						#$k -= 1;
						if ($delete eq "y") {
							unlink($location) or warn "could not remove file: $!";
						}
					}
	
				}
			}
			# Delete the temporary playlist
			$tmp_playlist->Delete();
			# Restore the previously selected playlist
			$iTunes->BrowserWindow->LetProperty('SelectedPlaylist', $old_playlist);
		}
	}
}

1;
