###############################################################################
#
# itunes_remove_notfound.pl
#
# This script will remove "lost" tracks from the iTunes Music library.
#
# Copyright (C) 2007  Robert Jacobson
# written by: Robert Jacobson (http://mysite.verizon.net/teridon/itunesscripts)
# Last Updated: 30 July 2007
# Version 1.1
#
# This script is GPL v2.  see http://www.gnu.org/copyleft/gpl.html
#
#	1.1 Update loop logic to start at end of list and step backwards through items.
#		Hopefully this will avoid a possible race condition
#
###############################################################################

sub itunes_remove_notfound {
	
	use strict;
	use Win32::OLE;
	
	## Create the OLE Object
	my $iTunes = Win32::OLE->new('iTunes.Application') or die Win32::OLE->LastError();
	
	# Always use the Library playlist
	my $playlist = $iTunes->LibraryPlaylist;
	
	my $tracks = $playlist->Tracks;
	my $num_tracks = $tracks->Count();
	print "\tThere are $num_tracks tracks in your Library\n";

	my $old_playlist = $iTunes->BrowserWindow->SelectedPlaylist;
	my $tmp_playlist = $iTunes->CreatePlaylist("Temporary Playlist $$");

	# Select the temporary playlist to increase the speed of the script
	# For some reason I have to use LetProperty() instead of
	# $iTunes->BrowserWindow->{SelectedPlaylist} = $tmp_playlist;
	$iTunes->BrowserWindow->LetProperty('SelectedPlaylist', $tmp_playlist);
	
	print "***\n*** Leave the Temporary Playlist selected -- the script will run faster\n***\n";
	
	my $deleted_tracks = 0;

	# Get all the tracks in the playlist
	for (my $k = $tracks->Count ; $k > 0 ; $k-- ) {
		#print "num: " , $num_tracks , " Count: ",  $tracks->Count , " k: ", $k , "\n";
		my $track = $tracks->Item($k);
		my $track_kind = $track->Kind();
	
		if ($track_kind == 1) {
			# File
			my $songname = $track->Name();
			my $artist = $track->Artist();
			my $path = $track->Location();
			if ($path eq "") {
				print "\t#### Removing $k : $songname, $artist\n";
				$track->Delete();
				$deleted_tracks++;
			}
		}
	}
	$tmp_playlist->Delete();
	# Restore the previously selected playlist
	$iTunes->BrowserWindow->LetProperty('SelectedPlaylist', $old_playlist);

	print "Removed $deleted_tracks dead tracks\n";
}

1;
