###############################################################################
#
# itunes_scripts.pl
#
# This script lets you run various scripts to fix your iTunes problems.
#
# Copyright (C) 2009  Robert Jacobson
# written by: Robert Jacobson (http://www.pobox.com/~teridon/itunesscripts)
# Last Updated: 2009 Sep 11
# Version 2.2
#
# 2.0 This version accepts the script to run as a command line argument.  Meant
#		to be used from AutoIT GUI
# 2.1 Add script mm_rating_to_itunes.pl
# 2.2 Modify mm_rating_to_itunes to work with MediaMonkey's new, retarded, POPM values
#
# This script is GPL v2.  see http://www.gnu.org/copyleft/gpl.html
#
###############################################################################

use File::Basename;
my $PROGNAME = basename($0);
my $VERSION = "2.2";
my $AUTHOR = "Robert Jacobson";
my $HOMEPAGE = "http://www.pobox.com/~teridon/";
my $YEAR = 2009;
my $GNU_URL = "http://www.gnu.org/copyleft/gpl.html";

{
	print
	"**************************************************************\n" .
	"$PROGNAME version $VERSION\n" . 
	"Copyright (C) $YEAR $AUTHOR\n" .
	"Visit $HOMEPAGE for updates\n" . 
	"$PROGNAME comes with ABSOLUTELY NO WARRANTY;\n".
	"This is free software, and you are welcome\n" .
	"to redistribute it under certain conditions\n" .
	"for details see $GNU_URL.\n" .
	"**************************************************************\n" .
	"\n"
	;
}

use strict;
use warnings;
use Win32::OLE;

#my $iTunes = Win32::OLE->new('iTunes.Application', sub {$_[0]->Quit;}) or die Win32::OLE->LastError();
my $iTunes = Win32::OLE->new('iTunes.Application') or die Win32::OLE->LastError();
use sigtrap 'handler', \&quit, 'normal-signals';

require 'itunes_add_m3u_playlist.pl';
require 'itunes_album_and_singles_playlist.pl';
require 'itunes_album_playlists.pl';
require 'itunes_artist_album_listing.pl';
require 'itunes_backup_playlists.pl';
require 'itunes_find_by_location.pl';
require 'itunes_find_by_name.pl';
require 'itunes_insert_artwork.pl';
require 'itunes_ipod_playlist.pl';
require 'itunes_ipod_playlist_to_itunes.pl';
require 'itunes_no_artwork_playlist.pl';
require 'itunes_preferences_fix.pl';
require 'itunes_remove_by_location.pl';
require 'itunes_remove_by_playlist.pl';
require 'itunes_remove_duplicates.pl';
require 'itunes_remove_notfound.pl';
require 'itunes_vidkind.pl';
require 'itunes_change_showname.pl';
require 'mm_rating_to_itunes.pl';

my %script_list = (
	itunes_album_playlists => "creates playlists for every album it finds in your Library",
	itunes_album_and_singles_playlist => 
		"looks in the entire music library and makes two playlists - \n" .
		"\"Albums\" and \"Singles\".  The \"Albums\" list will contain\n" . 
		"all the songs from albums that have >= 6 tracks from that album.\n" .
		"Similarly, the \"Singles\" playlist will have all the songs that\n" .
		"have <= 3 tracks from that album.",
	itunes_add_m3u_playlist =>
		"find all .m3u playlists in the given directory and add the contents to iTunes.",
	itunes_artist_album_listing => 
		"creates a text file on your Desktop called \"albumlisting.txt\" which\n" .
		"lists all the albums for each artist found in your Library.",
	itunes_backup_playlists =>
		"backup selected (or all) playlists in iTunes to a text file (one text file\n" .
		"per playlist). Files are placed in a subdirectory of the same location\n" .
		"as the script called \"Playlist Backups\".",
	itunes_change_showname =>
		"Change the Show name of the selected video files to whatever you want",
	itunes_find_by_location =>
		"find all tracks in the user-selected playlist whose FILE PATH matches\n" .
		"a user-supplied pattern (regex).  Matching tracks are added to  a new\n" .
		"playlist called \"Location\"",
	itunes_find_by_name =>
		"find all tracks in the user-selected playlist whose SONG NAME matches\n" .
		"a user-supplied pattern (regex).  Matching tracks are added to  a new\n" .
		"playlist called \"Song Name\"",
	itunes_insert_artwork =>
		"will embed downloaded artwork into the ID3 tag for the selected playlist",
	itunes_ipod_playlist =>
		"takes all the songs on the iPod and creates a new playlist in iTunes\n" .
		"called \"iPod Songs\" containing all the songs on the iPod.  Tracks\n" .
		"must already be in your Library",
	itunes_ipod_playlist_to_itunes =>
		"attempt to recreate selected playlists from your iPod in iTunes.\n" .
		"The songs must already be in your Library.",
	itunes_no_artwork_playlist =>
		"find all the tracks in the selected playlist that do not have artwork\n" .
		"and put them in a playlist called \"No Artwork\"",
	itunes_preferences_fix =>
		"restore the settings in iTunes so that it will ask for confirmation\n" .
		"when deleting either a song from the Library or a non-empty playlist.\n".
		"*****NOTE: only tested with iTunes 7.0, 7.1, 7.2, and 7.4!",
	itunes_remove_by_location =>
		"find all tracks in the user-selected playlist whose file path matches\n" .
		"a user-supplied pattern (regex).  Matching tracks are removed from\n" .
		"the playlist.  The files remain in the Library (unless you select the\n" .
		"the Library playlist)"	,
	itunes_remove_by_playlist =>
		"remove all songs from your iTunes Library based on the playlist you\n" .
		"selected.  It will also optionally remove those files from your hard\n" .
		"drive.",
	itunes_remove_duplicates =>
		"remove duplicates from the user-specified playlist(s).\n" .
		"Duplicates are determined by exact file/path name",
	itunes_remove_notfound =>
		"remove \"lost\" tracks from the iTunes Music library.",
	itunes_vidkind =>
		"change the video type of files in a the playlist you select",
	mm_rating_to_itunes =>
		"Import MediaMonkey ratings into iTunes",
);

#for my $script ( keys %script_list) {
	#require $script . ".pl";
#}

MAIN:

if ($#ARGV == 0) {
	print "Running $ARGV[0]\n";
	eval ( $ARGV[0] );
	print "Press enter to exit";
	chomp ( my $trash = <STDIN>);
} else {
	my $j = 1;
	my @scripts;
	foreach my $script ( sort keys %script_list ) {
	# 	print "$j)	$script\n$script_list{$script}\n\n";
		print "$j)	$script\n";
		$scripts[$j] = $script;
		$j++;
	}
	
	
	my $ans = "foo";
	while ( $ans !~ /^(q|\d+)$/ ) {
		print "\nSelect script to describe [or enter q to exit] ";
		chomp ($ans = <STDIN>);
	}
	
	if ($ans =~ /^q/i) {
		quit();
	}
	
	print "\n\n";
	
	# Print Description
	print "$scripts[$ans] Description:\n";
	print "$script_list{$scripts[$ans]}\n\n";
	
	print "Run it? ";
	
	chomp (my $ans2 = <STDIN>);
	
	if ($ans2 =~ /^y/i ) {
		print "Running $scripts[$ans]...\n\n****Type Ctrl-C at any time to cancel****\n\n";
		eval ( $scripts[$ans] );
		print "\n\nScript $scripts[$ans] Finished, returning to Main menu\n\n";
		goto MAIN;
	} else {
		print "Returning to Main menu\n\n";
		goto MAIN;
	}
}
quit();

sub quit 
{
	# This destroys the object
	undef $iTunes;
	exit;
}
