###############################################################################
#
# itunes_set_playeddate.pl
#
# This script will allow you to set the Last Played Date for the selected
# iTunes tracks.  Why anyone would want to do so is beyond me, but this guys
# wants to:
# http://discussions.apple.com/thread.jspa?messageID=3843705&#3843705
#
# Copyright (C) 2007  Robert Jacobson
# written by: Robert Jacobson (http://mysite.verizon.net/teridon/itunesscripts)
# Last Updated: 28 Jan 2007
# Version 1.0
#
# This script is GPL v2.  see http://www.gnu.org/copyleft/gpl.html
#
###############################################################################

use File::Basename;
my $PROGNAME = basename($0);
my $VERSION = "1.0";
my $AUTHOR = "Robert Jacobson";
my $HOMEPAGE = "http://mysite.verizon.net/teridon/";
my $YEAR = 2007;
my $GNU_URL = "http://www.gnu.org/copyleft/gpl.html";

{
	print
	"**************************************************************\n" .
	"$PROGNAME version $VERSION, Copyright (C) $YEAR $AUTHOR\n" .
	"Visit $HOMEPAGE for updates\n" . 
	"$PROGNAME comes with ABSOLUTELY NO WARRANTY;\n".
	"This is free software, and you are welcome\n" .
	"to redistribute it under certain conditions\n" .
	"for details see $GNU_URL.\n" .
	"**************************************************************\n" .
	"\n"
	;
}

use Win32::OLE;
use Win32::OLE::Variant;
use Win32::OLE::NLS qw(:LOCALE :TIME);

use strict;


use sigtrap 'handler', \&quit, 'normal-signals';

## Create the OLE Object
my $iTunes = Win32::OLE->new('iTunes.Application') or die Win32::OLE->LastError();

my $input;
while (not input_valid($input) ){
	print "Input date in format M/D/YYYY hh:mm : ";
	chomp ($input = <STDIN>);
}

my $v = Variant(VT_DATE, $input);

my $tracks = $iTunes->SelectedTracks;

for (my $i = 1; $i <= $tracks->Count ; $i++) {
	my $track = $tracks->Item($i);
	$track->{'PlayedDate'} = $v;
}

sub input_valid {
	my $input = shift;
	return 0 if ( not $input );
	if ( $input !~ m|^\d\d?/\d\d?/\d{4,4} \d\d?:\d\d?$|) {print "invalid format\n"; return 0;}
	my ($month, $day, $year, $hour, $minute) = split(/[\/: ]/, $input);
	#print "$month, $day, $year, $hour, $minute\n";
	if ($month < 1 || $month > 12 || ( $month - int($month) > 0 )) {print "invalid month\n"; return 0;}
 	if ($day < 1 || $day > 31 || ( $day - int($day) > 0 )) {print "invalid day\n"; return 0;}
 	if ($year < 1990 || $year > 2035 || ( $year - int($year) > 0 )) {print "invalid year\n"; return 0;}
 	if ($hour < 0 || $hour > 23 || ( $hour - int($hour) > 0 )) {print "invalid hour\n"; return 0;}
 	if ($minute < 0 || $minute > 59 || ( $minute - int($minute) > 0 )) {print "invalid minute\n"; return 0;}
	
	return 1;
}

# END
quit();

sub quit 
{
	# This destroys the object
	undef $iTunes;
	exit;
}
